<?php

/**
 * PDO database access classes
 *
 * @package objSQL
 * @version 3.2.0
 * @author MT Jordan <mtjo62@gmail.com>
 * @copyright 2013
 * @license zlib/libpng
 * @link http://objsql.sourceforge.net 
 */

 
/*************************************************************************************************************
 * Begin database connection/utility class
 ************************************************************************************************************/
 
  
class pdo_connection extends obj_access 
{
    /**********************************************
     * Internal variables
     *********************************************/

    /**
     * Database connection object
     *
     * @access private
     * @var    mixed
     */
    private $obj_connection = null;
    
    /**
     * Database connection information
     *
     * @access private
     * @var array
     */
    private $obj_datasource;
	
	 /**
     * Database type
     *
     * @access private
     * @var    str
     */
    private $obj_db_type;

    /**********************************************
     * Class methods
     *********************************************/

    /**
     * Constructor
     *
     * @access public
     * @param  array $datasource
     */
    public function __construct( $datasource, $dbtype )
    { 
        $this->obj_datasource = $datasource;
		$this->obj_db_type = $dbtype;
    }

    /**
     * Returns database connection object
     *
     * @access protected
     * @return mixed
     */
    protected function obj_db_connection()
    {
		if ( $this->obj_db_type == 'sqlite3' )
		{
			$obj_DSN = "sqlite:{$this->obj_datasource[4]}";
		}
		else
		{
			if ( ( array_key_exists( 5, $this->obj_datasource ) ) && is_numeric( trim( $this->obj_datasource[5] ) ) )
			{  
				if ( $this->obj_db_type == 'sqlsrv' )
					$obj_DSN = "sqlsrv:Server={$this->obj_datasource[1]},{$this->obj_datasource[5]};Database={$this->obj_datasource[4]}";
				else
					$obj_DSN = "{$this->obj_db_type}:host={$this->obj_datasource[1]};port={$this->obj_datasource[5]};dbname={$this->obj_datasource[4]}";
			}		
			else
			{
				if ( $this->obj_db_type == 'sqlsrv' )
					$obj_DSN = "sqlsrv:Server={$this->obj_datasource[1]};Database={$this->obj_datasource[4]}";
				else
					$obj_DSN = "{$this->obj_db_type}:host={$this->obj_datasource[1]};dbname={$this->obj_datasource[4]}";			
			}
		}
		
		$db_connection = new PDO( $obj_DSN, $this->obj_datasource[2], $this->obj_datasource[3] ); 
		
		return ( is_object( $db_connection ) ) ? $this->obj_connection = $db_connection : false;
	}

    /**
     * Closes connection to database server
     *
     * @access protected
     * @return bool
     */
    protected function obj_db_close()
    {
        return $this->obj_connection = null;
    }
	
	/**
     * Returns error flag for current connection instance
     *
     * @access protected
     * @return bool
     */
    protected function obj_db_error()
    {
        return ( $this->obj_connection->errorInfo()[1] != '' ) ? true : false;
	}
    
    /**
     * Escapes string data for database insertion
     *
     * @access protected
     * @param  mixed $data
     * @return mixed
     */
    protected function obj_db_escape_data( $data )
    {
		if ( $this->obj_db_type == 'mysql' )
			return str_replace( "'", "\'", $data );
		else
			return str_replace( "'", "''", $data );
    }
    
    /**
     * Returns error message for current connection instance
     *
     * @access protected
     * @return mixed
     */
    protected function obj_db_message()
    {
        return ( $this->obj_db_error() ) ? $this->obj_connection->errorInfo()[2] : null;
    }
    
    /**
     * Returns database server information
     *
     * @access protected
     * @return array
     */
    protected function obj_db_info()
    {
		if ( $this->obj_db_type == 'cubrid' ) 
		{
			$obj_get_info = $this->obj_connection->query( "SELECT COLLATION('abc')" );
			$obj_encoding = $obj_get_info->fetch( PDO::FETCH_NUM )[0];
		}
		elseif ( $this->obj_db_type == 'mysql' ) 
		{
			$obj_get_info = $this->obj_connection->query( "SHOW TABLE STATUS FROM {$this->obj_datasource[4]}" );
			$obj_encoding = $obj_get_info->fetch( PDO::FETCH_NUM )[14];
		}
		elseif ( $this->obj_db_type == 'pgsql' ) 
		{		
			$obj_get_info = explode( ';', $this->obj_connection->getAttribute( constant( 'PDO::ATTR_SERVER_INFO' ) ) );
			$obj_get_encoding = explode( ':', $obj_get_info[1] );
			$obj_encoding = $obj_get_encoding[1];
		}
		elseif ( $this->obj_db_type == 'sqlsrv' ) 
		{
			$obj_get_info = $this->obj_connection->query( "SELECT collation_name FROM sys.databases WHERE NAME='{$this->obj_datasource[4]}'" );
			$obj_encoding = $obj_get_info->fetch( PDO::FETCH_OBJ )->collation_name;
		}
		else
			$obj_encoding = 'UTF8';
				
		$obj_get_info = null;
		
		return array( $this->obj_connection->getAttribute( constant( 'PDO::ATTR_SERVER_VERSION' ) ), $obj_encoding, $this->obj_datasource[4] );
    }
	
	/**
     * Returns row count for named table with arguments
	 * Returns -1 if undetermined or failure
     *
     * @access protected
	 * @param  str $table
     * @param  str $cols
     * @param  str $where 
     * @return int
     */
    protected function obj_db_rowcount( $table, $cols=false, $where=false )
    {
		$query_cols  = ( !trim( $cols ) ) ? '*' : $cols;
        $query_where = ( !trim( $where ) ) ? '' : "WHERE $where"; 
		
		$query_stmt = $this->obj_connection->query( "SELECT COUNT($query_cols) FROM $table $query_where" );
		
		if ( !is_object( $query_stmt ) )
			return -1;
				
        while ( $num_rows = $query_stmt->fetch( PDO::FETCH_NUM ) )
			$rowcount = ( $num_rows[0] >= 0 ) ? $num_rows[0] : -1;
		
		$query_stmt = null;
		
		return $rowcount;
	}
}


/*************************************************************************************************************
 * End database connection class
 ************************************************************************************************************/

 
/*************************************************************************************************************
 * Begin database statement class
 ************************************************************************************************************/
 

class pdo_statement
{ 
    /**********************************************
     * Internal variables
     *********************************************/

    /**
     * Database connection object
     *
     * @access private
     * @var    mixed
     */
    private $obj_connection;
	
	/**
     * Database driver type
     *
     * @access private
     * @var    str
     */
    private $obj_db_type;

    /**
     * Query string
     *
     * @access private
     * @var    str
     */
    private $obj_query;
    
    /**********************************************
     * Class methods
     *********************************************/

    /**
     * Constructor
     *
     * @access public
     * @param  str   $query
     * @param  mixed $connection
	 * @param  str   $dbtype
     */
    function __construct( $query, $connection, $dbtype )
    {
        $this->obj_connection = $connection->obj_connection;
		$this->obj_db_type = $dbtype;
        $this->obj_query = $query;
    }
    
    /**
     * Executes general query and returns resultset object
     *
     * @access public
     * @return mixed
     */
    public function obj_query_execute()
    {
        $query_stmt = $this->obj_connection->query( $this->obj_query );
        
        return ( is_object( $query_stmt ) ) ? new pdo_resultset( $query_stmt, $this->obj_db_type ) : false;
    }
 }


/*************************************************************************************************************
 * End database statement class
 ************************************************************************************************************/
 
 
/*************************************************************************************************************
 * Begin database prepared statement class
 ************************************************************************************************************/
 

class pdo_prepare
{
    /**********************************************
     * Internal variables
     *********************************************/

    /**
     * Database connection object
     *
     * @access private
     * @var    mixed
     */
    private $obj_connection;
	
	/**
     * Database type
     *
     * @access private
     * @var    str
     */
    private $obj_db_type;
	
	/**
     * Set obj_bind parameter counter
     *
     * @access private
     * @var    int
     */ 
    private $obj_parameter_cnt = 0;

   /**
     * Prepared statement instance
     *
     * @access private
     * @var    bool
     */
    private $obj_prepare_instance;
    
    /**********************************************
     * Class methods
     *********************************************/

    /**
     * Constructor
     *
     * @access public
     * @param  mixed $connection
	 * @param  str   $query 
	 * @param  str   $dbtype
     */
    function __construct( $query, $connection, $dbtype ) 
    {
        $this->obj_connection = $connection->obj_connection;
		$this->obj_db_type = $dbtype;
        $this->obj_prepare_init( $query );
    }
    
    /**
     * Sets parameters and parameter types for prepared statement
     *
     * @access public
     * @param  mixed $param
     */
    public function obj_bind( $param )
    {
        $this->obj_parameter_cnt++;
        
        return $this->obj_prepare_instance->bindParam( $this->obj_parameter_cnt, $param );
    }
    
    /**
     * Destroys prepared statement object
     *
     * @access public
     * @return bool 
     */
    public function obj_close_statement()
    {
        $this->obj_parameter_cnt = 0;
        $this->obj_prepare_instance->closeCursor();
		
		return ( !is_object( $this->obj_prepare_instance = null ) ) ? true : false;
    }
   
    /**
     * Binds parameters, executes prepared statement and returns resultset object
     *
     * @access public
     * @return mixed
     */
    public function obj_execute()
    {
        $query_stmt = $this->obj_prepare_instance->execute();
                
        if ( $query_stmt )
			return new pdo_resultset( $this->obj_prepare_instance, $this->obj_db_type );
        else
            return false;
    }
    
    /**
     * Frees resultset memory from prepared statement object and resets binding parameters
     *
     * @access public
     * @return bool 
     */
    public function obj_free_statement()
    {
        $this->obj_parameter_cnt = 0;
        
        return $this->obj_prepare_instance->closeCursor();
    }
    
    /**
     * Returns prepared statement instance
     *
     * @access private
     * @param  str $query
     * @return bool
     */
    private function obj_prepare_init( $query )
    {
        return $this->obj_prepare_instance = $this->obj_connection->prepare( $query );
    }
}


/*************************************************************************************************************
 * End database prepared statement class
 ************************************************************************************************************/

 
/*************************************************************************************************************
 * Begin database resultset class
 ************************************************************************************************************/
 

class pdo_resultset
{
    /**********************************************
     * Internal variables
     *********************************************/

	/**
     * Database type
     *
     * @access private
     * @var    str
     */
    private $obj_db_type;
	
    /**
     * Query record
     *
     * @access private
     * @var    array
     */
    private $obj_record = array();

    /**
     * Query resultset object
     *
     * @access private
     * @var    mixed
     */
    private $obj_result = false;

    /**********************************************
     * Class methods
     *********************************************/
    /**
     * Constructor
     *
     * @access public
     * @param  mixed $result
	 * @param  mixed $dbtype
     */
    public function __construct( $result, $dbtype )
    {
		$this->obj_result = $result;
		$this->obj_db_type = $dbtype;
    }

    /**
     * Returns number of affected rows from insert/delete/update query
     * Returns -1 if undetermined or failure
     *
     * @access public
     * @return int
     */
    public function obj_affected_rows()
    {
        $affected_rows = $this->obj_result->rowCount();
        
        return ( $affected_rows >= 0 ) ? $affected_rows : -1;
    }

    /**
     * Returns resultset object as associative array
     *
     * @access public
     * @return mixed
     */
    public function obj_fetch_assoc()
    {
        $result = $this->obj_result->fetch( PDO::FETCH_ASSOC );
            
        return ( is_array( $result ) && $result !== false ) ? $this->obj_record = $result : null;
    }

    /**
     * Returns resultset object as numeric array
     *
     * @access public
     * @return mixed
     */
    public function obj_fetch_num()
    {
        $result = $this->obj_result->fetch( PDO::FETCH_NUM );
            
        return ( is_array( $result ) && $result !== false ) ? $this->obj_record = $result : null;
    }
	
	/**
     * Returns resultset object as object
     *
     * @access public
     * @return mixed
     */
     
    public function obj_fetch_object()
    {
        $result = $this->obj_result->fetch( PDO::FETCH_OBJ );
            
        return ( is_object( $result ) && $result !== false ) ? $this->obj_record = $result : null;
    }

    /**
     * Returns resultset record
     *
     * @access public
     * @param  mixed $field
     * @return mixed
     */
    public function obj_field( $field )
    {
        if ( $this->obj_result )
        {
            //get_magic_quotes deprecated in php 5.4 - added for backwards compatibility
            return ( get_magic_quotes_runtime() ) ? stripslashes( $this->obj_record[$field] ) : $this->obj_record[$field];
        }
    }

    /**
     * Frees resultset memory and destroys resultset object
     *
     * @access public
     * @return bool
     */
    public function obj_free_result()
    {
        $this->obj_record = array();
            
        return ( !is_object( $this->obj_result = null ) ) ? true : false;
    }
    
    /**
     * Returns number of fields from query
     * Returns -1 if undetermined or failure
     *
     * @access public
     * @return int
     */
    public function obj_num_fields()
    {
        $num_cols = $this->obj_result->columnCount();
        
        return ( $num_cols >= 0 ) ? $num_cols : -1; 
    }

   /**
     * Returns number rows from query
     * Returns -1 if undetermined or failure 
     * 
     * @access public
     * @return int
     */
    public function obj_num_rows()
    {
		if (  $this->obj_db_type == 'sqlite3' || $this->obj_db_type == 'sqlsrv' )
			$num_rows = count( $this->obj_result->fetchAll() );
		else
			$num_rows = $this->obj_result->rowCount(); 
        
		return ( $num_rows >= 0 ) ? $num_rows : -1;
	}
}


/*************************************************************************************************************
 * End database resultset class
 ************************************************************************************************************/

 
/*************************************************************************************************************
 * Begin database transaction class
 ************************************************************************************************************/


class pdo_transaction
{
	/**********************************************
     * Internal variables
     *********************************************/
	
	/**
     * Database connection instance
     *
     * @access private
     * @var    mixed
     */
    private $obj_connection;
	
	/**
     * Database type
     *
     * @access private
     * @var    mixed
     */
    private $obj_db_type;
    
    /**********************************************
     * Class methods
     *********************************************/

    /**
     * Constructor
     *
     * @access public
     * @param  mixed $connection
	 * @param  str   $dbtype
     */
    public function __construct( $connection, $dbtype )
    {
        $this->obj_connection = $connection;
		$this->obj_db_type = $dbtype;

        //turn off autocommit
        $this->obj_connection->beginTransaction();
    }
    
    /**
     * Commits transaction for current transaction instance
     *
     * @access public
     * @return bool
     */
    public function obj_commit()
    {
        return $this->obj_connection->commit();
    }

    /**
     * Rollbacks transaction for current transaction instance
     *
     * @access public
     * @param  str $savepoint
     * @return bool
     */
    public function obj_rollback( $savepoint=false )
    {
        if ( !$savepoint ) 
            $rollback = $this->obj_connection->rollBack();
        else 
		{
			if ( $this->obj_db_type == 'cubrid' )
				$rollback = $this->obj_connection->query( "ROLLBACK WORK TO SAVEPOINT $savepoint" );
			elseif ( $this->obj_db_type == 'sqlite3' )
				$rollback = $this->obj_connection->query( "ROLLBACK TO $savepoint" );
			elseif ( $this->obj_db_type == 'sqlsrv' )
				$rollback = $this->obj_connection->query( "ROLLBACK TRANSACTION $savepoint" );
			else
				$rollback = $this->obj_connection->query( "ROLLBACK TO SAVEPOINT $savepoint" );
		}
        
        return ( $rollback || is_object( $rollback ) ) ? true : false;
    }

    /**
     * Creates transaction savepoint for current transaction instance
     *
     * @access public
     * @param  str $savepoint
     * @return bool
     */
    public function obj_savepoint( $savepoint )
    {
		if ( $this->obj_db_type == 'sqlsrv' )
			return ( is_object( $this->obj_connection->query( "SAVE TRANSACTION $savepoint" ) ) ) ? true : false;
		else
			return ( is_object( $this->obj_connection->query( "SAVEPOINT $savepoint" ) ) ) ? true : false;
    }
}


/*************************************************************************************************************
 * End database transaction class
 ************************************************************************************************************/
 

?>